/*
 * Skeleton-code behorende bij het college Netwerken, opleiding Informatica,
 * Universiteit Leiden.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

#include <unistd.h>

#include "variables.h"
#include "common.h"

/* Stop-and-wait ARQ, sender part. */


/* Main loop */
static void
main_loop(void)
{
  /* The number of packets that have been transmitted and acknowledged by B. */
  int pkt_tx = 0;

  while (pkt_tx < SR_N_PACKETS)
    {
      /* Send packet. */
      struct packet_t packet;

      memset(&packet, 0, sizeof(struct packet_t));

      packet.type = PACKET_TYPE_DATA;
      packet.seq_number = pkt_tx % 2;
      snprintf(packet.mystring, MYSTRING_SIZE, "A to B part #%d", pkt_tx);
      packet.length = strlen(packet.mystring) + 1;

      packet_buffer_put(&outgoing_buffer, &packet, PACKET_LOSS_PERC);
      msleep(PACKET_DELAY_TIME_A);

      /* Wait for ACK or until timer expires. */
      while (true)
        {
          struct packet_t recv_packet;

          bool received = packet_buffer_get(&incoming_buffer, &recv_packet,
                                            ARQ_TIMEOUT);
          msleep(PACKET_DELAY_TIME_A);

          /* Discard the frame if it is damaged. */
          if (frame_damaged(PACKET_DAMAGED_PERC))
            continue;

          if (!received)
            {
              /* Timeout expired: retransmit */
#ifdef DEBUG
              puts("Timer expired: retransmit.");
#endif
              packet_buffer_put(&outgoing_buffer, &packet,
                                PACKET_LOSS_PERC);
              msleep(PACKET_DELAY_TIME_A);
              continue;
            }
          else if (recv_packet.type == PACKET_TYPE_RR)
            {
              if (recv_packet.seq_number == (pkt_tx + 1) % 2)
                /* Correct packet has been acknowledged. */
                break;
#ifdef DEBUG
              puts("Wrong packet acknowledged.");
#endif
            }
          else
            fprintf(stderr, "error: invalid packet type ...\n");
        }

      /* Prepare for next frame: increment counter, system delay */
      pkt_tx++;
    }
}

int
main(int argc, char **argv)
{
  /* Check command line arguments */
  if (argc < 3)
    {
      fprintf(stderr, "usage: %s <A-port> <B-ip> <B-port>\n", argv[0]);
      fprintf(stderr, "\n  'A-port' specifies the port A should listen at.");
      fprintf(stderr, "\n  'B-ip' specifies the IP address of B.");
      fprintf(stderr, "\n  'B-port' specifies the port B is listening at.\n");
      return -1;
    }

  int A_port, B_port;
  in_addr_t B_inaddr;
  if (!check_arguments(argv, &A_port, &B_inaddr, &B_port))
    return -1;

  run(A_port, B_inaddr, B_port, main_loop);

  return 0;
}
