/*
 * Skeleton-code behorende bij het college Netwerken, opleiding Informatica,
 * Universiteit Leiden.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>

#include <unistd.h>

#include "variables.h"
#include "common.h"

/* Stop-and-wait ARQ, receiver part. */


/* Main loop */
static void
main_loop(void)
{
  /* The number of packets that have been received and acknowledged by B. */
  int pkt_rx = 0;

  while (pkt_rx < SR_N_PACKETS)
    {
      /* Wait for packet. */
      struct packet_t packet;

      packet_buffer_get(&incoming_buffer, &packet, 0);
      msleep(PACKET_DELAY_TIME_B);

      /* Discard frame if it is damaged. */
      if (frame_damaged(PACKET_DAMAGED_PERC))
        continue;

      if (packet.type == PACKET_TYPE_DATA)
        {
          if (packet.seq_number == pkt_rx % 2)
            {
              /* Accept packet. */
              puts(packet.mystring);

              pkt_rx++;
            }
          /* Otherwise, discard the packet, but do send an ACK. */
#ifdef DEBUG
          else
            puts("Received packet out of order: discard.");
#endif
        }

      /* Send ACK with packet number we are ready to receive. */
      struct packet_t reply_packet;

      memset(&reply_packet, 0, sizeof(struct packet_t));

      reply_packet.type = PACKET_TYPE_RR;
      reply_packet.seq_number = pkt_rx % 2;

      packet_buffer_put(&outgoing_buffer, &reply_packet,
                        PACKET_LOSS_PERC);
      msleep(PACKET_DELAY_TIME_B);
    }
}

int
main(int argc, char **argv)
{
  /* Check command line arguments */
  if (argc < 3)
    {
      fprintf(stderr, "usage: %s <B-port> <A-ip> <A-port>\n", argv[0]);
      fprintf(stderr, "\n  'B-port' specifies the port B should listen at.");
      fprintf(stderr, "\n  'A-ip' specifies the IP address of A.");
      fprintf(stderr, "\n  'A-port' specifies the port A is listening at.\n");
      return -1;
    }

  int B_port, A_port;
  in_addr_t A_inaddr;
  if (!check_arguments(argv, &B_port, &A_inaddr, &A_port))
    return -1;

  run(B_port, A_inaddr, A_port, main_loop);

  return 0;
}
